@@ -11,25 +11,22 @@ import traceback
 import sqlite3
 
 
-def csvjoin_main():
+def _csvjoin_main():
     parser = argparse.ArgumentParser(description="CSV query in SQL. Yonghang Wang, wyhang@gmail.com, 2021")
     parser.add_argument( "-t", "--csv", "--table", dest="tables", action="append",default=list(), help="specify csv files. '[alias=]csvfile'")
-    parser.add_argument( "-v", "--view", dest="views", action="append", default=list(),help="view full definition. can be a backdoor if needed.")
     parser.add_argument( "-i", "--index", dest="indexes", action="append",default=list(), help="index. tbl(c1,c2,...)")
     parser.add_argument( "-d", "--db", "--database",dest="db", default=":memory:",  help="database name. default in memory.")
     parser.add_argument( "-q", "--sql", "--query",dest="sql", default=None,  help="SQL stmt or file containing sql query")
+    parser.add_argument( "-a", "--adhoc", dest="adhoc", action="append", default=list(),help="adhoc DDL/DML such as view full definition.")
     parser.add_argument( "-J", "--json", dest="json", action="store_true", default=False, help="dump result in JSON",)
     parser.add_argument( "-X", "--debug", dest="debug", action="store_true", default=False, help="debug mode",)
+    parser.add_argument( "--table-creation-mode", dest="tablemode", default="replace", help="if_exists{fail,replace,append}, default 'replace'",)
     args = parser.parse_args()
     
     def _x(s) :
         if args.debug :
             print("# "+s,file=sys.stderr,flush=True)
 
-    #if not ( (args.tables or args.db != ":memory:")  and args.sql ) :
-    #    print("# pls specify csv files(-t) and/or sql(-q).",file=sys.stderr,flush=True)
-    #    sys.exit(-1)
-
     con = sqlite3.connect(args.db)
     cur = None
 
@@ -40,7 +37,7 @@ def csvjoin_main():
             tbname = csvfile.replace(".csv","").replace(".","_")
         _x("loading table {} from {}".format(tbname,csvfile))
         df = pandas.read_csv(os.path.expanduser(csvfile)) 
-        df.to_sql(tbname, con, if_exists="replace")
+        df.to_sql(tbname, con, if_exists=args.tablemode)
         con.commit()
 
     def randname(n) :
@@ -55,7 +52,7 @@ def csvjoin_main():
         cur.execute(stmt)
         con.commit()
 
-    for vstmt in args.views :
+    for vstmt in args.adhoc :
         if not cur :
             cur = con.cursor()
         _x(vstmt)
@@ -72,15 +69,24 @@ def csvjoin_main():
         df = pandas.read_sql_query(sql, con)
     
         if args.json :
-            print(df.to_json(orient="records"))
+            print(df.to_json(orient="records"),flush=True)
         else :
+            if df.empty :
+                print("# empty set.",file=sys.stderr,flush=True)
+                return
             pandas.set_option("max_columns",None)
             pandas.set_option("max_rows",None)
             pandas.options.display.width = 0
-            print(df)
+            print(df,flush=True)
+
+
+def csvjoin_main():
+    try :
+        _csvjoin_main()
+    except :
+        exitinfo = traceback.format_exc().splitlines()[-1]
+        if "SystemExit: 0" not in exitinfo :
+            print(exitinfo,file=sys.stderr,flush=True)
 
 if __name__ == "__main__":
-    try :
-        csvjoin_main()
-    except :
-        print(traceback.format_exc().splitlines()[-1],file=sys.stderr,flush=True)
+    csvjoin_main()
